/**
 * Clase del objeto Resorte.
 */
public class Spring extends PhysicsElement  {
   private static int id=0;  // Spring identification
   protected float restLength; //Largo del resorte en reposo
   protected final float stiffness; //Coeficiente de elasticidad
   protected Block block_a, block_b; //Bloques en cada extremo
   /**
    * Implementacion y especificacion de los atributos del objeto resorte.
    * @param restLength largo en reposo del resorte.
    * @param stiffness coeficiente de elasticidad.
    */
   public Spring(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      block_a = block_b = null;
   }
    /**
    * Metodo para vincular el extremo A al bloque.
    * @param a bloque a vincular.
    */
   public void attachBlock_a (Block a) {
      block_a = a;
      block_a.attachSpring(this);
   }
   /**
    * Metodo para remover del extremo A el bloque.
    */
   public void detachBlock_a(){
      if (block_a == null) return;
      block_a.detachSpring(this);
      block_a=null;
   }
   /**
    * Metodo para vincular el extremo B al bloque.
    * @param b bloque a vincular.
    */
   public void attachBlock_b (Block b) {
      block_b = b;
      block_b.attachSpring(this);
   }
   /**
    * Metodo para remover del extremo B el bloque.
    */
   public void detachBlock_b(){
      if (block_b == null) return;
      block_b.detachSpring(this);
      block_b=null;
   }
    /**
    * Metodo para obtener el vector de posicion del extremo A del resorte si tiene vinculado un bloque.
    * @return vector de posicion.
    */
   protected Vector2D getPositionA() {
      if (block_a != null)
         return block_a.getPosition();
      return null;
   }
    /**
    * Metodo para obtener el vector de posicion del extremo B del resorte si tiene vinculado un bloque.
    * @return vector de posicion.
    */
   protected Vector2D getPositionB() {
      if (block_b != null)
         return block_b.getPosition();
      return null;
   }
   /**
    * Metodo para obtener el vector que describe la distancia entre los extremos del resorte.
    * @return vector.
    */
   protected Vector2D getVector() {
      Vector2D b = getPositionB();
      Vector2D a = getPositionA();
      if (a== null || b==null) return new Vector2D();
      else return b.minus(a);
   }
   /**
    * Metodo para obtener la fuerza que ejerce el resorte sobre el bloque conectado en su extremo.
    * @param block bloque conectado al extremo del resorte.
    * @return vector de fuerza para el bloque.
    */
   public Vector2D getForce(Block block) {
      Vector2D force;
      if ((block_a == null) || (block_b == null))
         return new Vector2D(); 
      Vector2D springVector = getVector();
      double stretch = springVector.module() - restLength;
      force = springVector.unitary().times(stretch*stiffness);
      if (block == block_a) return force;
      if (block == block_b) return force.times(-1);
      else return new Vector2D();
   }
    /**
    * Metodo no utilizado.
    * @param delta_t
    * @param gravity
    */
   public void computeNextState(double delta_t, double gravity) {
   }
   /**
    * Metodo no utilizado.
    * @param delta_t
    */
   public void checkNext(double delta_t){
   }
   /**
    * Metodo no utilizado.
    * @param damp
    */
   public void updateState(boolean damp){
   }
   /**
    * Metodo para obtener la informacion del resorte.
    * @return descripcion.
    */
   public String getDescription() {
      return "Spring#"+ getId()+": "+Vector2D.getDescription()+
            ","+Vector2D.getDescription()+",";
   }
   /**
    * Metodo para obtener la informacion de la posicion
    * @return String con la descripcion.
    */
   public String getState() {
      String s;
      if (block_a != null) 
         s = block_a.getPosition()+",";
      else if (block_b != null)
              s = block_b.getPosition().minus(new Vector2D(restLength,0))+",";
            else  
              s = new Vector2D()+",";  // up to here left end is determined 
      if (block_a != null)
         s += block_a.getPosition();
      else if (block_b != null)
               s += block_b.getPosition().plus(new Vector2D(restLength,0));
            else
               s += new Vector2D();
      return s+",";
   }
}
